#!/bin/sh

#  installer.sh
#  Actifile
#
#  Created by Oleg Kulchytskyi on 9/10/19.
#  Copyright © 2019 Actifile. All rights reserved.

function usage
{
    echo "Usage:"
    echo "    $1 install <SRC_PATH> <DST_PATH> <INST_KEY> <DEV_UUID> <UNINST_KEY>"
    echo "    $1 uninstall <DST_PATH>"
    exit 1
}

function log_step
{
    echo -e "========================================"
    echo -e "$1"
    echo -e "========================================\n"
}

function copy_artifact
{
    echo -e "Copy Artifact \n$SRC_PATH/$1 \n$DST_PATH/$1 \n"
    cp -r "$SRC_PATH/$1" "$DST_PATH/$1"
    chmod -R $2 "$DST_PATH/$1"
    chown -R root:wheel "$DST_PATH/$1"
}

#
# USAGE: install_daemon <PLIST_PATH> <EXEC_PATH> <IN_BUNDLE>
#
function install_daemon
{
    if [ $# -lt 2 ]
    then
        echo "$FUNCNAME -- not enough input parameters"
        return 1
    fi

    local DAEMON_PATH="/Library/LaunchDaemons/$1"
    local DAEMON_EXEC="$DST_PATH/$2$3"
    local DAEMON_LOG="$DST_PATH/${1%.*}_0.log"

    log_step "Installing Daemon $DAEMON_PATH"

    if [ -f "$DAEMON_PATH" ]
    then
        launchctl bootout system "$DAEMON_PATH" &>/dev/null
    fi

    copy_artifact "$2" 755

    #
    # NOTE: Copy daemon plist
    #
    cp -r "$SRC_PATH/$1" "$DAEMON_PATH"
    chmod -R 644 "$DAEMON_PATH"
    chown -R root:wheel "$DAEMON_PATH"

    echo "$DAEMON_EXEC"
    plutil -replace "ProgramArguments" -json "[\"$DAEMON_EXEC\"]" "$DAEMON_PATH"
    plutil -replace "StandardOutPath" -string "$DAEMON_LOG" "$DAEMON_PATH"
    plutil -replace "StandardErrorPath" -string "$DAEMON_LOG" "$DAEMON_PATH"

    log_step "Launching Daemon $DAEMON_PATH"

    launchctl bootstrap system "$DAEMON_PATH"
    if [ $? -ne 0 ]
    then
        echo "Failed to launch Daemon $DAEMON_PATH"
        return 1
    fi
}

#
# USAGE: uninstall_daemon <PLIST_PATH>
#
function uninstall_daemon
{
    if [ $# -ne 1 ]
    then
        echo "$FUNCNAME -- not enough input parameters"
        return 1
    fi

    local DAEMON_PATH="/Library/LaunchDaemons/$1"
    local DAEMON_NAME="${1%.*}"

    launchctl bootout system "$DAEMON_PATH" &> /dev/null
    local BOOTOUT_STATUS=$?
    # status 36 -- EINPROGRESS
    if [[ $BOOTOUT_STATUS != 0 && $BOOTOUT_STATUS != 36 ]]
    then
        echo "Failed to stop Daemon $DAEMON_PATH -- $BOOTOUT_STATUS"
        return 1
    fi

    if [[ $BOOTOUT_STATUS == 36 ]]
    then
        while [[ `launchctl list $DAEMON_NAME &> /dev/null` == 0 ]]
        do
            sleep 1
        done
    fi

    rm "$DAEMON_PATH"
}

#
# USAGE: install_agent <PLIST_PATH> <EXEC_PATH> <IN_BUNDLE>
#
function install_agent
{
    if [ $# -ne 3 ]
    then
        echo "$FUNCNAME -- not enough input parameters"
        return 1
    fi

    local AGENT_PATH="$LAUNCH_AGENTS_PATH/$1"
    local AGENT_EXEC="$DST_PATH/$2$3"
    local AGENT_LOG="$DST_PATH/${1%.*}_0.log"
    echo $AGENT_LOG

    log_step "Installing Agent $AGENT_PATH"

    if [ -f "$AGENT_PATH" ]
    then
        launchctl bootout gui/$INST_UID $AGENT_PATH
    fi

    cp -r "$SRC_PATH/$1" "$AGENT_PATH"
    chmod -R 644 "$AGENT_PATH"
    chown -R :staff "$AGENT_PATH"

    copy_artifact "$2" 755

    plutil -replace "ProgramArguments" -json "[\"$AGENT_EXEC\"]" "$AGENT_PATH"
    plutil -replace "StandardOutPath" -string "$AGENT_LOG" "$AGENT_PATH"
    plutil -replace "StandardErrorPath" -string "$AGENT_LOG" "$AGENT_PATH"
    
    touch "$AGENT_LOG"
    chown -R :staff "$AGENT_LOG"
    chmod -R 775 "$AGENT_LOG"

    log_step "Launching Agent $AGENT_PATH"

    ERROR_FLAG=1

    for INST_UID in $(dscacheutil -q user | grep "5[0-9][0-9]" | awk '{print $2}')
    do
        launchctl bootstrap gui/$INST_UID "$AGENT_PATH"
        BOOTSTRAP_STATUS=$?
        if [ $BOOTSTRAP_STATUS -ne 0 ]
        then
            echo "Failed to launch Agent $AGENT_PATH -- $INST_UID -- $BOOTSTRAP_STATUS "
        else
            ERROR_FLAG=0
        fi
    done

    if [ $ERROR_FLAG -ne 0 ]; then
        exit 1
    fi

}

#
# USAGE: uninstall_agent <PLIST_PATH>
#
function uninstall_agent
{
    if [ $# -ne 1 ]
    then
        echo "$FUNCNAME -- not enough input parameters"
        return 1
    fi

    local AGENT_PATH="$LAUNCH_AGENTS_PATH/$1"
    local AGENT_NAME="${1%.*}"
    
    for INST_UID in $(dscacheutil -q user | grep "5[0-9][0-9]" | awk '{print $2}')
    do
        launchctl bootout gui/$INST_UID "$AGENT_PATH" &> /dev/null
    done

    rm "$AGENT_PATH"

    for USER_DIR in /Users/*
    do
        AGENT_PLIST_PATH="$USER_DIR/$AGENT_PATH"
        if [ -f "$AGENT_PLIST_PATH" ]
        then
            echo "Removing $AGENT_PLIST_PATH"
            rm "$AGENT_PLIST_PATH"
        fi
    done
}

#
# USAGE: install_actifile <SRC_PATH> <DST_PATH> <INST_KEY> <DEV_UUID> <UNINST_KEY>
#
function install_actifile
{
    if [ $# != 5 ]
    then
        echo "$FUNCNAME -- not enough input parameters"
        usage $0
    fi

    if [ ! -d "$1" ]
    then
        echo "Source path $1 -- Does not exists"
        usage $0
    fi


    rm "/Library/LaunchDaemons/com.actifile.uninstalld.plist"

    local SRC_PATH="${1%%/}"
    local DST_PATH="${2%%/}"
    local INST_KEY=$3
    local DEV_UUID=$4
    local UNINST_KEY=$5
    local LAUNCH_AGENTS_PATH="/Library/LaunchAgents"

    log_step "Creating $DST_PATH"

    mkdir -p "$DST_PATH"
    if [ $? -ne 0 ]
    then
        echo "Failed to create DST_PATH -- $DST_PATH"
        return 1
    fi
    chown :staff "$DST_PATH"

    log_step "Creating $LAUNCH_AGENTS_PATH"

    mkdir -p "$LAUNCH_AGENTS_PATH"
    if [ $? -ne 0 ]
    then
        echo "Failed to create LAUNCH_AGENTS_PATH -- $LAUNCH_AGENTS_PATH"
        return 1
    fi
    chown :staff $LAUNCH_AGENTS_PATH


    #
    # NOTE: Copy and prepare settings
    #
    copy_artifact "settings.json" 644

    OUTPUT=`sed 's/${DEV_UUID}/'$DEV_UUID'/g' "$DST_PATH/settings.json"`
    echo "$OUTPUT" > "$DST_PATH/settings.json"
    OUTPUT=`sed 's/${INST_KEY}/'$INST_KEY'/g' "$DST_PATH/settings.json"`
    echo "$OUTPUT" > "$DST_PATH/settings.json"
    OUTPUT=`sed 's/${INST_USER}/'$INST_USER'/g' "$DST_PATH/settings.json"`
    echo "$OUTPUT" > "$DST_PATH/settings.json"
    OUTPUT=`sed 's/${UNINST_KEY}/'$UNINST_KEY'/g' "$DST_PATH/settings.json"`
    echo "$OUTPUT" > "$DST_PATH/settings.json"

    #
    # NOTE: Generate Updater ID. Make positive int in range 0...1000000
    #
    UPDATER_ID=$((`od -An -N4 -i /dev/urandom | sed 's/-//g'` % 1000000))
    OUTPUT=`sed 's/${UPDATER_ID}/'$UPDATER_ID'/g' "$DST_PATH/settings.json"`
    echo "$OUTPUT" > "$DST_PATH/settings.json"
    
    #
    # NOTE: copy installer script for remote uninstallation support
    #
    SCRIPT_NAME="${0##*/}"
    copy_artifact "$SCRIPT_NAME" 755

    install_daemon "com.actifile.afmond.plist" "afmond.app" "/Contents/MacOS/afmond"
    install_daemon "com.actifile.updaterd.plist" "updaterd"
    install_agent "com.actifile.UserAgent.plist" "ActifileUserAgent.app" "/Contents/MacOS/ActifileUserAgent"

}

#
# USAGE: uninstall_actifile <DST_PATH>
#
function uninstall_actifile
{
    if [ $# != 1 ]
    then
        echo "$FUNCNAME -- not enough input parameters"
        usage $0
    fi

    if [ ! -d "$1" ]
    then
        echo "Destination path $1 -- Does not exists"
        usage $0
    fi

    local DST_PATH="${1%%/}"
    local LAUNCH_AGENTS_PATH="/Library/LaunchAgents"

    log_step "Uninstalling afmond"
    uninstall_daemon "com.actifile.afmond.plist"

    log_step "Uninstalling updaterd"
    uninstall_daemon "com.actifile.updaterd.plist"

    log_step "Uninstalling UserAgent"
    uninstall_agent "com.actifile.UserAgent.plist"

    log_step "Removing files"

    rm -rf "$DST_PATH"/afmond.app
    rm -f "$DST_PATH"/com.actifile.afmond*.log
    rm -f "$DST_PATH"/updaterd
    rm -f "$DST_PATH"/com.actifile.updaterd*.log
    rm -f "$DST_PATH"/installer.sh
    
    rm -f "$DST_PATH"/UserAgent
    rm -f "$DST_PATH"/com.actifile.UserAgent*.log
    rm -f "$DST_PATH"/Cache.db
    rm -rf "$DST_PATH"/Requests
    rm -f "$DST_PATH"/*.csv
    rm -f "$DST_PATH"/state.json

    rm -rf "$DST_PATH"/ActifileUserAgent.app
}

case $1 in
"install")
    if [ $# != 6 ]
    then
        echo "$0 -- not enough input parameters"
        usage $0
    fi

    install_actifile "$2" "$3" "$4" "$5" "$6"

    if [ $? -ne 0 ]
    then
        uninstall_actifile "$2"
        usage $0
    fi

    ;;

"uninstall")
    if [ $# != 2 ]
    then
        echo "$0 -- not enough input parameters"
        usage $0
    fi

    uninstall_actifile "$2"

    ;;

#
# WARN: This subcommand only for internal usage
#
"selfuninstall")
    if [ $# != 3 ]
    then
        echo "$0 -- not enough input parameters"
        exit 1
    fi

    if [ ! -f "$3" ]
    then
        echo "$3 -- file does not exists"
        exit 1
    fi

    uninstall_actifile "$2"
    log_step "Running selfuninstall stop -- $3"
    launchctl bootout system "$3"
    
    rm "/Library/LaunchDaemons/com.actifile.uninstalld.plist"
    
    ;;

*)
    usage $0
    ;;
esac
